# the plugin code was generated by this script
# you should not change the plugin code directly, but this script
# note: this script only creates objects in your workspace,
# *EXCEPT* for the last call, see below.

local({
# set the output directory to overwrite the actual plugin
output.dir <- tempdir()
overwrite <- TRUE

require(rkwarddev)

about.info <- rk.XML.about(
	name="rk.ANOVA",
	author=c(
		person(given="Meik", family="Michalke",
			email="meik.michalke@hhu.de", role=c("aut","cre"))),
	about=list(desc="RKWard GUI to conduct ANOVAs (using the ez package), pairwise t-Tests and plot interactions.",
		version="0.01-5", url="http://rkward.sf.net"),
	dependencies=list(rkward.min="0.5.7"),
	package=list(c(name="ez"), c(name="sciplot"))
	)

############
## ANOVA
############
anova.drp.design <- rk.XML.radio("Design", options=list(
		"Between subjects"=c(val="between"),
		"Within subjects (repeated measures)"=c(val="within", chk=TRUE),
		"Mixed"=c(val="mixed")
	))
var.select <- rk.XML.varselector(label="Select data")
var.data <- rk.XML.varslot(label="Data (must be data.frame)", source=var.select, required=TRUE, classes="data.frame")
var.dv <- rk.XML.varslot(label="Dependent variable", source=var.select, required=TRUE)
var.wid <- rk.XML.varslot(label="Case/subject identifier", source=var.select)
var.within <- rk.XML.varslot(label="Within subject variables", source=var.select, multi=TRUE)
var.between <- rk.XML.varslot(label="Between subject variables", source=var.select, multi=TRUE)
# var.observed

#diff
#reverse_diff
drp.vtype <- rk.XML.dropdown(
	label="Sums of squares type for unbalanced designs",
	options=list("Type 1"=c(val=1), "Type 2"=c(val=2, chk=TRUE), "Type 3"=c(val=3)))

# logic: only relevant for pure between designs
drp.white <- rk.XML.dropdown(
	label="Heteroscedasticity correction",
	options=list("None"=c(val="false", chk=TRUE), "hc3 (default)"=c(val="hc3"), "hc0"=c(val="hc0"), "hc1"=c(val="hc1"), "hc2"=c(val="hc2"), "hc4"=c(val="hc4")))

check.extrainfo <- rk.XML.cbox(label="Show sums of squares, raw likelihood ratios etc.", value="true")
check.aov <- rk.XML.cbox(label="Return 'aov' object", value="true", chk=TRUE)

var.chk.suppress <- rk.XML.cbox(label="Suppress package loading messages", value="true", chk=TRUE)
save.results <- rk.XML.saveobj("Save results to workspace", initial="anova.results")

tab1.data <- rk.XML.row(
		var.select,
		rk.XML.col(
			rk.XML.frame(var.data),
			rk.XML.frame(anova.drp.design),
			rk.XML.frame(var.dv, var.wid),
			rk.XML.frame(var.within, var.between)
		)
	)

tab2.options <- rk.XML.row(
		rk.XML.col(
			rk.XML.frame(drp.vtype),
			rk.XML.frame(drp.white),
			rk.XML.frame(rk.XML.col(check.extrainfo),rk.XML.col(check.aov)),
			rk.XML.stretch(),
			var.chk.suppress,
			save.results
		)
	)

full.dialog <- rk.XML.dialog(rk.XML.tabbook(label="ANOVA",
		tabs=list("Data"=tab1.data, "Options"=tab2.options)
	), label="ANOVA")

## logic section to tie the second varslot to the data.frame
	lgc.sect <- rk.XML.logic(
		rk.XML.connect(governor="current_object", client=var.data, set="available"),
		rk.XML.connect(governor=var.data, client=var.select, get="available", set="root"),
		anova.gov.data <- rk.XML.convert(sources=list(available=var.data), mode=c(notequals="")),
		anova.gov.between <- rk.XML.convert(sources=list(string=anova.drp.design), mode=c(equals="between")),
		anova.gov.within <- rk.XML.convert(sources=list(string=anova.drp.design), mode=c(equals="within")),
		anova.gov.mixed <- rk.XML.convert(sources=list(string=anova.drp.design), mode=c(equals="mixed")),
		anova.gov.show.bvars <- rk.XML.convert(sources=list(anova.gov.between, anova.gov.mixed), mode=c(or=""), id.name="lgc_bvars"),
		anova.gov.show.wvars <- rk.XML.convert(sources=list(anova.gov.within, anova.gov.mixed), mode=c(or=""), id.name="lgc_vvars"),
 		rk.XML.connect(governor=anova.gov.data, client=var.dv, set="enabled"),
 		rk.XML.connect(governor=anova.gov.data, client=var.between, set="enabled"),
 		rk.XML.connect(governor=anova.gov.data, client=var.within, set="enabled"),
 		rk.XML.connect(governor=anova.gov.data, client=var.wid, set="enabled"),
		rk.XML.connect(governor=anova.gov.show.wvars, client=var.wid, set="required"),
		rk.XML.connect(governor=anova.gov.show.bvars, client=var.between, set="visible"),
		rk.XML.connect(governor=anova.gov.show.wvars, client=var.within, set="visible"),
		rk.XML.connect(governor=anova.gov.show.bvars, client=var.between, set="required"),
		rk.XML.connect(governor=anova.gov.show.wvars, client=var.within, set="required")
	)

## JavaScript
js.calc <- rk.paste.JS(
	jsVarDv <- rk.JS.vars(var.dv, modifiers="shortname", join=", "),
	jsVarWid <- rk.JS.vars(var.wid, modifiers="shortname", join=", "),
	jsVarWithin <- rk.JS.vars(var.within, modifiers="shortname", join=", "),
	jsVarBetween <- rk.JS.vars(var.between, modifiers="shortname", join=", "),
	echo("anova.results <- ezANOVA(\n\t"),
	ite(var.data, echo("data=", var.data)),
	ite(var.dv, echo(",\n\tdv=.(", jsVarDv ,")")),
	ite(var.wid, echo(",\n\twid=.(", jsVarWid ,")")),
	ite(rkwarddev:::id(var.within, " != \"\" & ", anova.drp.design, " != \"between\""), echo(",\n\twithin=.(", jsVarWithin ,")")),
	ite(rkwarddev:::id(var.between, " != \"\" & ", anova.drp.design, " != \"within\""), echo(",\n\tbetween=.(", jsVarBetween ,")")),

	ite(rkwarddev:::id(drp.vtype, " != 2"), echo(",\n\ttype=", drp.vtype)),
	ite(rkwarddev:::id(drp.white, " != \"false\""), echo(",\n\twhite.adjust=\"", drp.white, "\"")),
	ite(check.extrainfo, echo(",\n\tdetailed=TRUE")),
	ite(check.aov, echo(",\n\treturn_aov=TRUE")),
	echo(")\n\n")
)

js.print <- rk.paste.JS(
	echo("rk.header(\"ANOVA\", level=3)\n"),
	echo("rk.print(anova.results[[\"ANOVA\"]])\n"),
	echo("if(\"Mauchly's Test for Sphericity\" %in% names(anova.results)){
		rk.header(\"Mauchly's Test for Sphericity\", level=3)
		rk.print(anova.results[[\"Mauchly's Test for Sphericity\"]])
	} else {}\n"),
	echo("if(\"Sphericity Corrections\" %in% names(anova.results)){
		rk.header(\"Sphericity Corrections\", level=3)
		rk.print(anova.results[[\"Sphericity Corrections\"]])
	} else {}\n"),
	echo("if(\"Levene's Test for Homgeneity\" %in% names(anova.results)){
		rk.header(\"Levene's Test for Homgeneity\", level=3)
		rk.print(anova.results[[\"Levene's Test for Homgeneity\"]])
	} else {}\n")
)

########
## pairwise t-tests
########
pt.var.selectVars <- rk.XML.varselector(label="Select data")
pt.tvar.data <- rk.XML.varslot(label="Response vector", source=pt.var.selectVars, required=TRUE)
pt.tvar.group <- rk.XML.varslot(label="Grouping vector or factor", source=pt.var.selectVars, required=TRUE)
pt.drp.adjust <- rk.XML.dropdown(
	label="Method for adjusting p values",
	options=list(
		"none"=c(val="none"),
		"Bonferroni"=c(val="bonferroni"), 
		"Holm"=c(val="holm", chk=TRUE),
		"Benjamini &amp; Hochberg (fdr)"=c(val="BH"),
		"Benjamini &amp; Yekutieli"=c(val="BY"),
		"Hochberg"=c(val="hochberg"),
		"Hommel"=c(val="hommel")
	))
pt.chk.poolSD <- rk.XML.cbox("Pooled SD for all groups", value="true")
pt.chk.paired <- rk.XML.cbox("Paired t-Tests", value="true", chk=TRUE)
pt.radio.altern <- rk.XML.radio("Alternative hypothesis",
		options=list(
			"Two-sided"=c(val="two.sided"),
			"First is greater"=c(val="greater"),
			"Second is greater"=c(val="less")
		)
	)
pt.full.dialog <- rk.XML.dialog(
	rk.XML.row(
		pt.var.selectVars,
		rk.XML.col(
			rk.XML.frame(pt.tvar.data, pt.tvar.group, label="Data"),
			rk.XML.frame(pt.drp.adjust, label="Alpha error correction"),
			rk.XML.frame(pt.chk.poolSD, pt.chk.paired),
			pt.radio.altern
		)
	), label="Pairwise t-Tests")

## logic
pt.lgc.sect <- rk.XML.logic(
  		pt.gov.pool <- rk.XML.convert(sources=list(state=pt.chk.poolSD), mode=c(equals="true")),
  		rk.XML.connect(governor=pt.gov.pool, client=pt.chk.paired, set="enabled", not=TRUE),
		pt.gov.pair <- rk.XML.convert(sources=list(state=pt.chk.paired), mode=c(equals="true")),
		rk.XML.connect(governor=pt.gov.pair, client=pt.chk.poolSD, set="enabled", not=TRUE)
	)

## JavaScript
pt.js.calc <- rk.paste.JS(
	echo("pair.t.results <- pairwise.t.test(\n\t"),
	ite(pt.tvar.data, echo("x=", pt.tvar.data)),
	ite(pt.tvar.group, echo(",\n\tg=", pt.tvar.group)),
	ite(pt.drp.adjust, echo(",\n\tp.adjust.method=\"", pt.drp.adjust, "\"")),
	ite(pt.chk.poolSD, echo(",\n\tpool.sd=TRUE")),
	ite(pt.chk.paired, echo(",\n\tpaired=TRUE")),
	ite(rkwarddev:::id(pt.radio.altern, " != \"two.sided\""), echo(",\n\talternative=\"", pt.radio.altern, "\"")),
	echo(")\n\n")
)

pt.js.print <- rk.paste.JS(
	echo("rk.print(pair.t.results)\n")
)

## make a whole component of the t-test
pttest.component <- rk.plugin.component("Pairwise t-Tests",
	xml=list(
		logic=pt.lgc.sect,
		dialog=pt.full.dialog),
	js=list(
		calculate=pt.js.calc,
		printout=pt.js.print),
	hierarchy=list("analysis", "means", "t-tests"),
	create=c("xml", "js"))

###########
## interaction plot
###########
ip.var.selectVars <- rk.XML.varselector(label="Select data")
ip.tvar.x <- rk.XML.varslot(label="Factor (x axis)", source=ip.var.selectVars, required=TRUE)
ip.tvar.response <- rk.XML.varslot(label="Response vector", source=ip.var.selectVars, required=TRUE)
ip.tvar.group <- rk.XML.varslot(label="Grouping factor (traces)", source=ip.var.selectVars)
ip.rad.plottype <- rk.XML.radio("Plot type", options=list(
	"Lineplot"=c(val="line", chk=TRUE),
	"Bargraph"=c(val="bar")))
ip.rad.ltype <- rk.XML.radio("Elements", options=list(
	"Lines + points"=c(val="b", chk=TRUE),
	"Lines only"=c(val="l"),
	"Points only"=c(val="p")))
ip.rad.btype <- rk.XML.radio("Bars", options=list(
	"Group bars"=c(val="group", chk=TRUE),
	"Split bars"=c(val="split")))
ip.chk.se <- rk.XML.cbox("Standard error", val="true", chk=TRUE)
ip.chk.legend <- rk.XML.cbox("Legend", val="true", chk=TRUE)
ip.inp.trace.label <- rk.XML.input("Legend label")
ip.plot.options <- rk.XML.embed(component="rkward::plot_options", button=TRUE, label="Generic plot options")
ip.preview <- rk.XML.preview()

## logic
ip.lgc.sect <- rk.XML.logic(
  		ip.gov.lineplot <- rk.XML.convert(sources=list(string=ip.rad.plottype), mode=c(equals="line")),
  		rk.XML.connect(governor=ip.gov.lineplot, client=ip.rad.ltype, set="visible"),
		rk.XML.connect(governor=ip.gov.lineplot, client=ip.rad.btype, set="visible", not=TRUE),
  		ip.gov.traces <- rk.XML.convert(sources=list(available=ip.tvar.group), mode=c(notequals="")),
		rk.XML.connect(governor=ip.gov.traces, client=ip.rad.btype, set="enabled"),
		rk.XML.connect(governor=ip.gov.traces, client=ip.chk.legend, set="enabled"),
  		ip.gov.leglabel <- rk.XML.convert(sources=list(ip.gov.traces, state=ip.chk.legend), mode=c(and="")),
		rk.XML.connect(governor=ip.gov.leglabel, client=ip.inp.trace.label, set="enabled"),
		rk.XML.connect(governor=ip.tvar.x, client=ip.plot.options, get="available", set="xvar"),
		rk.XML.connect(governor=ip.tvar.response, client=ip.plot.options, get="available", set="yvar"),
		rk.XML.set(ip.plot.options, set="allow_type", to=FALSE)
	)

ip.full.dialog <- rk.XML.dialog(
	rk.XML.row(
		ip.var.selectVars,
		rk.XML.col(
			rk.XML.frame(ip.tvar.x, ip.tvar.response, ip.tvar.group, label="Data"),
			rk.XML.stretch(),
			rk.XML.frame(
				rk.XML.row(
					rk.XML.col(ip.rad.plottype, ip.chk.se, ip.chk.legend, rk.XML.stretch()),
					rk.XML.col(ip.rad.ltype, ip.rad.btype, rk.XML.stretch())
				),
				ip.inp.trace.label,
				ip.plot.options,
				ip.preview)
		)
	), label="Interaction plot")

## JavaScript
ip.js.prnt <- 	rk.paste.JS(
	rk.comment("in case there are generic plot options defined:"),
	js.po.printout <- rk.JS.vars(ip.plot.options, modifiers="code.printout", check.modifiers=FALSE),
	ite("full", echo("rk.graph.on()\n")),
	ite(rkwarddev:::id(ip.rad.plottype, " == \"line\""),
		echo("\ttry({\n\t\tlineplot.CI("),
		echo("\ttry({\n\t\tbargraph.CI(")),
	ite(ip.tvar.x, echo("\n\t\t\tx.factor=", ip.tvar.x)),
 	ite(ip.tvar.response, echo(",\n\t\t\tresponse=", ip.tvar.response)),
 	ite(ip.tvar.group, echo(",\n\t\t\tgroup=", ip.tvar.group)),
 	ite(rkwarddev:::id(ip.rad.plottype, " == \"line\" & ", ip.rad.ltype, " != \"b\""), echo(",\n\t\t\ttype=\"", ip.rad.ltype, "\"")),
 	ite(rkwarddev:::id(ip.rad.plottype, " == \"bar\" & ", ip.rad.btype, " == \"split\""), echo(",\n\t\t\tsplit=TRUE")),
 	ite(rkwarddev:::id(ip.rad.plottype, " == \"line\" & !", ip.chk.legend, " & ", ip.tvar.group, " != \"\""), echo(",\n\t\t\tlegend=FALSE")),
 	ite(rkwarddev:::id(ip.rad.plottype, " == \"bar\" & ", ip.chk.legend, " == \"true\" & ", ip.tvar.group, " != \"\""), echo(",\n\t\t\tlegend=TRUE")),
 	ite(rkwarddev:::id(ip.chk.legend, " == \"true\" & ", ip.tvar.group, " != \"\" & ", ip.inp.trace.label, " != \"\""), echo(",\n\t\t\ttrace.label=\"", ip.inp.trace.label, "\"")),
 	ite(rkwarddev:::id("!", ip.chk.se), echo(",\n\t\t\tci.fun=function(x)c(mean(x, na.rm=TRUE), mean(x, na.rm=TRUE))")),
	echo(js.po.printout),
	echo(")\n\t})\n"),
 	ite("full", echo("rk.graph.off()\n"))
)

## make a whole component of the interaction plot
plot.component <- rk.plugin.component("Interaction plot",
	xml=list(
		logic=ip.lgc.sect,
		dialog=ip.full.dialog),
 	js=list(
		results.header="Interaction plot",
		require="sciplot",
 		doPrintout=ip.js.prnt),
	hierarchy=list("plots"),
	create=c("xml", "js"))


#############
## if you run the following function call, files will be written to tempdir!
#############
# this is where it get's serious, that is, here all of the above is put together into one plugin

rk.ANOVA.dir <<- rk.plugin.skeleton(
	about.info,
	path=output.dir,
	xml=list(
		logic=lgc.sect,
		dialog=full.dialog),
	js=list(
		require="ez",
		calculate=js.calc,
		printout=js.print,
		load.silencer=var.chk.suppress),
	pluginmap=list(name="ANOVA", hierarchy=list("analysis", "ANOVA")),
	components=list(pttest.component, plot.component),
	create=c("pmap", "xml", "js", "desc"),
	overwrite=overwrite,
	tests=FALSE,
#	edit=TRUE,
	load=TRUE)#,
#	show=TRUE)
})
